#include <stdio.h>
#include "xparameters.h"
#include "xtmrctr.h"

#define W 8

void initnumber(unsigned a[W], unsigned b[W]) {
  a[0] = 0x4e991e0a;
  a[1] = 0x03f661ca;
  a[2] = 0x567a0a95;
  a[3] = 0x74a66a43;
  a[4] = 0x7729901b;
  a[5] = 0xdd38f3b2;
  a[6] = 0x781af8da;
  a[7] = 0xd9a03845;
  b[0] = 0x2135fb0f;
  b[1] = 0x453d74e2;
  b[2] = 0x0a15664f;
  b[3] = 0xe2fe2773;
  b[4] = 0x548d3247;
  b[5] = 0x8a40a8d0;
  b[6] = 0xdded18b3;
  b[7] = 0x052e731f;
}

void mult64(unsigned a, 
	    unsigned b,
	    unsigned *lo, 
	    unsigned *hi) {

  // portable unsigned 64 bit multiplication
  // assumes 32 bit integers

  unsigned al = a & 0xffff;
  unsigned ah = (a >> 16) & 0xffff; 
  unsigned bl = b & 0xffff;
  unsigned bh = (b >> 16) & 0xffff; 

  unsigned p1 = al * bl;
  unsigned p2 = ah * bl;
  unsigned p3 = al * bh;
  unsigned p4 = ah * bh;

  unsigned q1 = p1 & 0xffff;
  unsigned q2 = ((p1 >> 16) & 0xffff) + (p2 & 0xffff) + (p3 & 0xffff);
  unsigned q3 = ((p2 >> 16) & 0xffff) + ((p3 >> 16) & 0xffff) + (p4 & 0xffff);

  if (q2 > 0xffff) {
    q3 = q3 + (q2 - 0xffff);
    q2 = q2 - 0xffff;
  }

  unsigned q4 = ((p4 >> 16) & 0xffff);
    
  if (q3 > 0xffff) {
    q4 = q4 + (q3 - 0xffff);
    q3 = q3 - 0xffff;
  }

  *lo = (q2 << 16) + q1;
  *hi = (q4 << 16) + q3;
}

void add32(unsigned a, 
	   unsigned b,
	   unsigned *q,
	   unsigned *y) {
  unsigned max = (a > b) ? a : b;
  *q += (a + b);
  if (*q < max)
    *y = 1;
  else
    *y = 0;
}

void inc64(unsigned *lo,
	   unsigned *hi) {
  unsigned p = *lo + 1;
  if (p < *lo)
    *hi = *hi + 1;
  *lo = p;
}

void widemult(unsigned a[W], 
	           unsigned b[W],
	           unsigned t[2*W]) {
  unsigned i, j, C, S;
  unsigned hi, lo;
  unsigned q, cy1, cy2;

  for (i=0; i<2*W; i++)
    t[i] = 0;

  for (i=0; i<W; i++) {
    C = 0;
    for (j=0; j<W; j++) {
      mult64(a[j], b[i], &lo, &hi);
      add32(lo, C, &q, &cy1);
      add32(t[i+j], q, &S, &cy2);
      C = hi;
      if (cy1)
	inc64(&S, &C);
      if (cy2)
	inc64(&S, &C);
      t[i+j] = S;
    }
    t[i+W] = C;
  }
}

void wideadd(unsigned a[W], 
	          unsigned b[W],
	          unsigned t[W]) {
  unsigned i, cy;

  for (i=0; i<W; i++)
    t[i] = 0;

  cy = 0;
  for (i=0; i<W; i++) {
    t[i] = cy;
    add32(a[i], b[i], &(t[i]), &cy);
  }
}

void widemagic(unsigned a[W], 
	            unsigned b[W],
	            unsigned t[W]) {
  unsigned tmp[2*W], btmp[W];
  unsigned i;
  for (i=0; i<W; i++)
    btmp[i] = b[i];
  for (i=0; i<(16384); i++) {
    widemult(a, btmp, tmp);
    wideadd(tmp, &(tmp[W]), btmp);
  }
  for (i=0; i<W; i++)
    t[i] = tmp[i];
}

//----------------------------------------------
//
//       MAKE NO CHANGES ABOVE THIS LINE
//
//   YOUR IMPLEMENTATION MUST USE THIS FUNCTION
//   CALL AS THE TOP-LEVEL FUNCTION
//

void  yourmagic(unsigned a[W], 
	             unsigned b[W],
	             unsigned t[W]) {
  
  widemagic(a, b, t);
  
}

//
//       MAKE NO CHANGES BELOW THIS LINE
//
//----------------------------------------------


int main() {
  unsigned a[W], b[W], tref[W], tdut[W];
  unsigned i, match;
  int refcycle, dutcycle;
  XTmrCtr T;
  
  XTmrCtr_Initialize(&T, XPAR_XPS_TIMER_0_DEVICE_ID);
  XTmrCtr_SetResetValue(&T, 0, 0);

  initnumber(a, b);

  XTmrCtr_Start(&T, 0);
  widemagic(a, b, tref);
  XTmrCtr_Stop(&T, 0);
  refcycle = XTmrCtr_GetValue(&T,0);
  
  initnumber(a, b);

  XTmrCtr_Start(&T, 0);
  yourmagic(a, b, tdut);
  XTmrCtr_Stop(&T, 0);
  dutcycle = XTmrCtr_GetValue(&T,0);

  xil_printf("REF OUT: %8x %8x %8x %8x %8x %8x %8x %8x\n",
 	                    tref[7],tref[6],tref[5],tref[4],
							  tref[3],tref[2],tref[1],tref[0]);

  xil_printf("DUT OUT: %8x %8x %8x %8x %8x %8x %8x %8x\n",
 	                    tdut[7],tdut[6],tdut[5],tdut[4],
							  tdut[3],tdut[2],tdut[1],tdut[0]);

  match = 1;
  for (i=0; i<W; i++)
     if (tdut[i] != tref[i])
	     match = 0;

  if (match)
     xil_printf("DUT matches REF!\n");
	  
  xil_printf("DUT Cycles %d\n", dutcycle);
  xil_printf("REF Cycles %d\n", refcycle);
  
  return 0;
}
